@@ -2,7 +2,9 @@
 # pylint: disable=unsupported-assignment-operation
 import json
 import shutil
+import sys
 from dataclasses import dataclass
+from io import BytesIO
 from functools import partial
 from multiprocessing import Pool
 from zlib import crc32
@@ -14,7 +16,6 @@ import oead.aamp
 from oead.yaz0 import compress
 import pymsyt
 from rstb import ResourceSizeTable, SizeCalculator
-from rstb.util import write_rstb
 
 from . import (
     AAMP_EXTS,
@@ -60,6 +61,13 @@ class BuildParams:
     titles: set
     table: StockHashTable
     warn: bool
+    strict: bool
+
+    def warning(self, msg: str):
+        if self.strict:
+            raise RuntimeError(f"ERROR: {msg}")
+        elif self.warn:
+            print(f"WARNING: {msg}")
 
 
 def _should_rstb(f: Path) -> bool:
@@ -146,8 +154,9 @@ def _build_yml(f: Path, params: BuildParams):
                 )
             }
     except Exception as e:  # pylint: disable=broad-except
-        print(f"Failed to build {f.relative_to(params.mod).as_posix()}: {e}")
-        return {}
+        raise RuntimeError(
+            f"Failed to build {f.relative_to(params.mod).as_posix()}. {e}"
+        )
     if params.verbose:
         print(f"Built {f.relative_to(params.mod).as_posix()}")
     return rv
@@ -156,6 +165,7 @@ def _build_yml(f: Path, params: BuildParams):
 LINK_MAP = {
     3293308145: "AIProgram/*.baiprog",
     2851261459: "AISchedule/*.baischedule",
+    1241489578: "AnimationInfo/*.baniminfo",
     1767976113: "Awareness/*.bawareness",
     713857735: "BoneControl/*.bbonectrl",
     2863165669: "Chemical/*.bchemical",
@@ -303,12 +313,11 @@ def _build_actor(link: Path, params: BuildParams):
                             phys_source / "Ragdoll" / rg_path
                         )
                     else:
-                        if params.warn:
-                            print(
-                                f"WARNING: Havok ragdoll file Physics/Ragdoll/{rg_path} not found "
-                                f"for actor {actor_name}. Ignore if intentionally using a file not "
-                                "in the actor pack."
-                            )
+                        params.warning(
+                            f"Havok ragdoll file Physics/Ragdoll/{rg_path} not found "
+                            f"for actor {actor_name}. Ignore if intentionally using a file not "
+                            "in the actor pack."
+                        )
                 if types.params["use_support_bone"].v:
                     sb_path = str(
                         phys.lists["ParamSet"]
@@ -321,12 +330,11 @@ def _build_actor(link: Path, params: BuildParams):
                             phys_source / "SupportBone" / sb_path
                         )
                     else:
-                        if params.warn:
-                            print(
-                                f"WARNING: Havok support bone file Physics/SupportBone/{sb_path} "
-                                f"not found for actor {actor_name}. Ignore if intentionally using "
-                                "a file not in the actor pack."
-                            )
+                        params.warning(
+                            f"Havok support bone file Physics/SupportBone/{sb_path} "
+                            f"not found for actor {actor_name}. Ignore if intentionally using "
+                            "a file not in the actor pack."
+                        )
                 if types.params["use_cloth"].v:
                     cloth_path = str(
                         phys.lists["ParamSet"]
@@ -340,12 +348,11 @@ def _build_actor(link: Path, params: BuildParams):
                             phys_source / "Cloth" / cloth_path
                         )
                     else:
-                        if params.warn:
-                            print(
-                                f"WARNING: Havok cloth file Physics/Cloth/{cloth_path} not found "
-                                f"for actor {actor_name}. Ignore if intentionally using a file not "
-                                "in the actor pack."
-                            )
+                        params.warning(
+                            f"Havok cloth file Physics/Cloth/{cloth_path} not found "
+                            f"for actor {actor_name}. Ignore if intentionally using a file not "
+                            "in the actor pack."
+                        )
                 if types.params["use_rigid_body_set_num"].v > 0:
                     for _, rigid in (
                         phys.lists["ParamSet"].lists["RigidBodySet"].lists.items()
@@ -359,13 +366,12 @@ def _build_actor(link: Path, params: BuildParams):
                                     phys_source / "RigidBody" / rigid_path
                                 )
                             else:
-                                if params.warn:
-                                    print(
-                                        "WARNING: Havok rigid body file "
-                                        f"Physics/RigidBody/{rigid_path} not found for actor "
-                                        f"{actor_name}. Ignore if intentionally using a file not in"
-                                        " the actor pack."
-                                    )
+                                params.warning(
+                                    "Havok rigid body file "
+                                    f"Physics/RigidBody/{rigid_path} not found for actor "
+                                    f"{actor_name}. Ignore if intentionally using a file not in"
+                                    " the actor pack."
+                                )
                         except KeyError:
                             continue
         for name, path in files.items():
@@ -376,8 +382,9 @@ def _build_actor(link: Path, params: BuildParams):
             ):
                 modified = True
     except FileNotFoundError as e:
-        print(
-            f'Failed to build actor "{actor_name}": Could not find linked file "{e.filename}".'
+        params.warning(
+            f'Failed to build actor "{actor_name}". Could not find linked file "'
+            f'{Path(e.filename).relative_to(params.out)}".'
         )
         return {}
     _, sb = pack.write()
@@ -455,7 +462,7 @@ def _build_sarc(d: Path, params: BuildParams):
             else compress(sb)
         )
     except:
-        print(f"Failed to build {d.relative_to(params.out).as_posix()}")
+        params.warning(f"Failed to build {d.relative_to(params.out).as_posix()}")
         return {}
     else:
         if params.verbose:
@@ -489,9 +496,11 @@ def build_mod(args):
     aoc = "aoc" if args.be else "01007EF00011F001/romfs"
     mod = Path(args.directory)
     if not ((mod / content).exists() or (mod / aoc).exists()):
-        print("The specified directory is not valid: no content or DLC folder found")
+        print(
+            "The specified directory does not appear to have a valid folder structure."
+        )
         print("Run `hyrule_builder build --help` for more information.")
-        exit(1)
+        sys.exit(2)
     out = mod.with_name(f"{mod.name}_build") if not args.output else Path(args.output)
     if out.exists():
         print("Removing old build...")
@@ -507,11 +516,18 @@ def build_mod(args):
         aoc=aoc,
         titles=set(args.title_actors.split(",")),
         table=StockHashTable(args.be),
-        warn=not args.suppress_warn,
+        warn=not args.no_warn,
+        strict=args.hard_warn,
     )
 
     print("Scanning source files...")
-    files = {f for f in mod.rglob("**/*") if f.is_file() and "ActorInfo" not in f.parts}
+    files = {
+        f
+        for f in mod.rglob("**/*")
+        if f.is_file()
+        and "ActorInfo" not in f.parts
+        and not str(f.relative_to(mod)).startswith(".")
+    }
     other_files = {f for f in files if f.suffix not in {".yml", ".msyt"}}
     yml_files = {f for f in files if f.suffix == ".yml"}
     f: Path
@@ -550,7 +566,11 @@ def build_mod(args):
         for f in yml_files:
             rvs.update(_build_yml(f, params))
     else:
-        results = p.map(partial(_build_yml, params=params), yml_files)
+        try:
+            results = p.map(partial(_build_yml, params=params), yml_files)
+        except RuntimeError as err:
+            print(err)
+            sys.exit(1)
         for r in results:
             rvs.update(r)
 
@@ -562,7 +582,11 @@ def build_mod(args):
             for a in actors:
                 rvs.update(_build_actor(a, params))
         else:
-            results = p.map(partial(_build_actor, params=params), actors)
+            try:
+                results = p.map(partial(_build_actor, params=params), actors)
+            except RuntimeError as err:
+                print(err)
+                sys.exit(1)
             for r in results:
                 rvs.update(r)
     for d in (out / content / "Physics").glob("*"):
@@ -629,7 +653,9 @@ def build_mod(args):
                             msg = f"Added {p}, set to {v}"
                     if args.verbose and msg:
                         print(msg)
-        write_rstb(table, str(rp.with_suffix(".srsizetable")), args.be)
+        buf = BytesIO()
+        table.write(buf, args.be)
+        rp.with_suffix(".srsizetable").write_bytes(compress(buf.getvalue()))
         if rp.exists():
             rp.unlink()
 
